# TC Replications Project, NW prep
# Benjamin Campbell
# Started 11/6/2018
# NW_DataPrep.R
# Prepare NW data for TNAM

# Have already done WC M and F

if(write.nw.data == TRUE){
  
  options(stringsAsFactors = FALSE)
  
  ############
  #
  # Load data
  #
  ############
  
  dv <- read.table(paste0(WD,"/Data/NW"),
                   header = TRUE)
  
  
  # Since we are also interested in predicting recidivism, we should construct a new variable
  # A value of 1 is if the member at any point recidivated (regardless of if 1 or 2)
  dv$recidivist <- rep(0, times = nrow(dv))
  dv$recidivist[dv$recidflag1 > 0] <- 1
  dv$recidivist[dv$recidflag == 0] <- 0
  table(dv$recidivist)
  
  ################
  #
  # Load Networks
  # 
  ################
  
  # Edge list for correct
  ledge <- read.table(paste0(WD,"/Data/NW-ledge"))
  colnames(ledge) <- c("eventDate", "sender", "receiver", "countEvents")
  length(unique(ledge$eventDate))
  length(unique(ledge$sender)) 
  length(unique(ledge$receiver)) 
  
  ledge <- ledge[ledge$sender <= 9999 | ledge$receiver <= 9999,]
  
  # Edge list for affirm
  sedge <- read.table(paste0(WD,"/Data/NW-sedge"))
  colnames(sedge) <- c("eventDate", "sender", "receiver", "countEvents")
  length(unique(sedge$eventDate)) 
  length(unique(sedge$sender)) 
  length(unique(sedge$receiver))
  
  sedge <- sedge[sedge$sender <= 9999 | sedge$receiver <= 9999,]
  
  ################
  #
  # Clean data
  # 
  ################
  
  dv$enterClean <- as.Date(as.character(dv$enter),format = '%m/%d/%Y')
  dv$exitClean <- as.Date(as.character(dv$exit),format = '%m/%d/%Y')
  
  library(stringr)
  dayEnter <- str_sub(dv$enterClean, start = -2)
  monthEnter <- str_sub(dv$enterClean, start = -5, end = -4)
  yearEnter <- str_sub(dv$enterClean, start = 1, end = 4)
  
  dayExit <- str_sub(dv$exitClean, start = -2)
  monthExit <- str_sub(dv$exitClean, start = -5, end = -4)
  yearExit <- str_sub(dv$exitClean, start = 1, end = 4)
  
  enterClean <- paste(yearEnter, monthEnter, dayEnter, sep = '') 
  exitClean <- paste(yearExit, monthExit, dayExit, sep = '')
  dv$enterClean <- enterClean
  dv$exitClean <- exitClean
  head(dv)
  dv$row <- row.names(dv)
  
  # Clean network dates
  sedge$eventDateClean <- as.Date(as.character(sedge$eventDate),format = '%m/%d/%Y')
  
  library(stringr)
  day <- str_sub(sedge$eventDateClean, start = -2)
  month  <- str_sub(sedge$eventDateClean, start = -5, end = -4)
  year <- str_sub(sedge$eventDateClean, start = 1, end = 4)
  
  sedge$eventDateClean <- paste(year, month, day, sep = '') 
  
  ledge$eventDateClean <- as.Date(as.character(ledge$eventDate),format = '%m/%d/%Y')
  
  day <- str_sub(ledge$eventDateClean, start = -2)
  month  <- str_sub(ledge$eventDateClean, start = -5, end = -4)
  year <- str_sub(ledge$eventDateClean, start = 1, end = 4)
  
  ledge$eventDateClean <- paste(year, month, day, sep = '') 
  
  # to make it easier, I'm going to make the wcid numbers more workable
  
  nwcid <- as.character(dv$NWID)
  dv$nwid_c <- as.character(dv$NWID)
  dv$nwid_new <- dv$NWID
  
  # Now, I need to check and see which observations are duplicated (as per Philip's suggestion)
  duplicatedObs <- dv[which(duplicated(dv$nwid_new) | duplicated(dv$nwid_new, fromLast = TRUE)),]
  # 13 individuals who came back
  
  repeatOffenders <- unique(duplicatedObs$nwid_new)
  
  if(length(repeatOffenders) > 0){
    sedge_repeat <- data.frame()
    ledge_repeat <- data.frame()
    dv_repeat <- data.frame()
    
    for(i in 1:length(repeatOffenders)){
      id <- repeatOffenders[i]
      records <- duplicatedObs[duplicatedObs$nwid_new == id,]
      
      # sort these repeat offenders so that itis first time in then last time, then label these
      records <- records[order(records$enterClean),] 
      timesRepeated <- 2
      newIds <- paste(id, ".", "Visit", 1:timesRepeated, sep = "")
      records$nwid_new <- newIds
      
      # then I want to do the same laeling according to sedge and ledge, both for senders and recievers, based upon timing
      sedge_repeat <- sedge[sedge$receiver == id | sedge$sender == id,]
      ledge_repeat <- ledge[ledge$receiver == id | ledge$sender == id,]
      
      temp_sedge <- data.frame()
      temp_ledge <- data.frame()
      
      for(t in 1:2){
        record_t <- records[t,]
        id_t <- newIds[t]
        
        sedge_t <- sedge_repeat[which(sedge_repeat$eventDateClean >= record_t$enterClean & sedge_repeat$eventDateClean <= record_t$exitClean),]
        sedge_t_senders <- which(sedge_t$sender == id)
        sedge_t$sender[sedge_t_senders] <- id_t
        sedge_t_receiver <- which(sedge_t$receiver == id)
        sedge_t$receiver[sedge_t_receiver] <- id_t
        
        ledge_t <- ledge_repeat[which(ledge_repeat$eventDateClean >= record_t$enterClean & ledge_repeat$eventDateClean <= record_t$exitClean),]
        ledge_t_senders <- which(ledge_t$sender == id)
        ledge_t$sender[ledge_t_senders] <- id_t
        ledge_t_receiver <- which(ledge_t$receiver == id)
        ledge_t$receiver[ledge_t_receiver] <- id_t
        
        temp_sedge <- rbind(temp_sedge, sedge_t)
        temp_ledge <- rbind(temp_ledge, ledge_t)
      }
      
      sedge_repeat <- rbind(sedge_repeat, temp_sedge)
      ledge_repeat <- rbind(ledge_repeat, temp_ledge)
      dv_repeat <- rbind(dv_repeat, records)
      # end loop
    }
    
    
    # Now, we must go through and update the sedge list to remove those repeated offenders and supplement with sedge_repeat and ledge_repeat
    sedge_remove <- c(which(sedge$sender %in% repeatOffenders), which(sedge$receiver %in% repeatOffenders))
    ledge_remove <- c(which(ledge$sender %in% repeatOffenders), which(ledge$receiver %in% repeatOffenders))
    
    sedge <- sedge[-c(sedge_remove),]
    ledge <- ledge[-c(ledge_remove),]
    
    sedge <- rbind(sedge, sedge_repeat)
    ledge <- rbind(ledge, ledge_repeat)
    
    # Do the same with dv
    dv <- dv[order(dv$enterClean),] 
    dv_remove <- c(which(dv$nwid_new %in% repeatOffenders))
    dv <- dv[-c(dv_remove),]
    dv <- rbind(dv, dv_repeat)
    
  }
  

  # Now, let's produce edgelists of corrections (ledge) and affirmations (sedge)
  affirm_el <- data.frame(Source = sedge$sender,
                          Target = sedge$receiver)
  
  
  correct_el <- data.frame(Source = ledge$sender,
                           Target = ledge$receiver)
  
  

  # Turn these into weighted edge lists
  library(plyr)
  affirm_el <- ddply(affirm_el,.(Source,Target),nrow)
  colnames(affirm_el)[3] <- "Weight"
  
  correct_el <- ddply(correct_el,.(Source,Target),nrow)
  colnames(correct_el)[3] <- "Weight"
  
  # Turn these into adjacency
  library(igraph)
  g <- graph.data.frame(affirm_el)
  affirm_adj <- get.adjacency(g,sparse=FALSE, attr="Weight")
  affirm_adj <- list(t1 = affirm_adj)
  
  g <- graph.data.frame(correct_el)
  correct_adj <- get.adjacency(g,sparse=FALSE, attr="Weight")
  correct_adj <- list(t1 = correct_adj)
  
  ### Now, I should loop through all of these and create a separate record for further instances of recidivism while
  # also accounting for their last visit to a TC
  ids <- unique(dv$nwid_new)
  
  # 
  new_df <- data.frame()
  for(i in 1:length(ids)){
    id <- ids[i]
    record <- dv[which(dv$nwid_new == id),]
    
    # Now I need to determine if this person has reciivated
    if(record$recidivist > 0){
      recid_list <- data.frame()
      if(is.na(record$recidflag1) == FALSE){
        record$recidClean1 <- as.Date(as.character(record$recidate1),format = '%m/%d/%Y')
        dayExit <- str_sub(record$recidClean1, start = -2)
        monthExit <- str_sub(record$recidClean1, start = -5, end = -4)
        yearExit <- str_sub(record$recidClean1, start = 1, end = 4)
        record$recidClean1 <- paste(yearExit, monthExit, dayExit, sep = '')
        
        first_reincarceration <- data.frame(
          wcid = record$nwid_new,
          enterClean = record$enterClean,
          recidDate = record$recidClean1,
          exitClean = record$exitClean,
          recidFlag = 1,
          strata = 0,
          success = record$success,
          age = record$age,
          lsi = record$lsi,
          black = record$black)
        
        recid_list <- rbind(recid_list, first_reincarceration)
      }
      
      if(is.na(record$recidflag2) == FALSE){
        record$recidClean2 <- as.Date(as.character(record$recidate2),format = '%m/%d/%Y')
        dayExit <- str_sub(record$recidClean2, start = -2)
        monthExit <- str_sub(record$recidClean2, start = -5, end = -4)
        yearExit <- str_sub(record$recidClean2, start = 1, end = 4)
        record$recidClean2 <- paste(yearExit, monthExit, dayExit, sep = '')
        
        second_reincarceration <- data.frame(
          wcid = record$nwid_new,
          enterClean = record$enterClean,
          recidDate = record$recidClean2,
          exitClean = record$exitClean,
          recidFlag = 1,
          strata = 1,
          success = record$success,
          age = record$age,
          lsi = record$lsi,
          black = record$black)
        
        recid_list <- rbind(recid_list, second_reincarceration)
      }
      
      if(is.na(record$recidflag3) == FALSE){
        record$recidClean3 <- as.Date(as.character(record$recidate3),format = '%m/%d/%Y')
        dayExit <- str_sub(record$recidClean3, start = -2)
        monthExit <- str_sub(record$recidClean3, start = -5, end = -4)
        yearExit <- str_sub(record$recidClean3, start = 1, end = 4)
        record$recidClean3 <- paste(yearExit, monthExit, dayExit, sep = '')
        
        third_reincarceration <- data.frame(
          wcid = record$nwid_new,
          enterClean = record$enterClean,
          recidDate = record$recidClean3,
          exitClean = record$exitClean,
          recidFlag = 1,
          strata = 2,
          success = record$success,
          age = record$age,
          lsi = record$lsi,
          black = record$black)
        
        recid_list <- rbind(recid_list, third_reincarceration)
      }
      
      if(is.na(record$recidflag4) == FALSE){
        record$recidClean4 <- as.Date(as.character(record$recidate4),format = '%m/%d/%Y')
        dayExit <- str_sub(record$recidClean4, start = -2)
        monthExit <- str_sub(record$recidClean4, start = -5, end = -4)
        yearExit <- str_sub(record$recidClean4, start = 1, end = 4)
        record$recidClean4 <- paste(yearExit, monthExit, dayExit, sep = '')
        
        fourth_reincarceration <- data.frame(
          wcid = record$nwid_new,
          enterClean = record$enterClean,
          recidDate = record$recidClean3,
          exitClean = record$exitClean,
          recidFlag = 1,
          strata = 3,
          success = record$success,
          age = record$age,
          lsi = record$lsi,
          black = record$black)
        
        recid_list <- rbind(recid_list, fourth_reincarceration)
      }
      
    } else {
      recid_list <- data.frame(
        wcid = record$nwid_new,
        enterClean = record$enterClean,
        recidDate = NA,
        exitClean = record$exitClean,
        recidFlag = 0,
        strata = 0,
        success = record$success,
        age = record$age,
        lsi = record$lsi,
        black = record$black)
    }
    new_df <- rbind(new_df, recid_list)
  }
  
  # Now I need to remove records for individuals who go back to the therapeutic community after recidivating 
  # loop through observations that end in .1 and .2, at which point I determine the instances of reincarceration
  # that occur following hte most recent visit
  
  mult_visits_Full <- rbind(new_df[grep("Visit1", new_df$nwid_new),], new_df[grep("Visit2", new_df$nwid_new),])
  
  mult_visits <- mult_visits_Full[mult_visits_Full$recidFlag == 1,]
  
  mult_visits_NOR <- mult_visits_Full[mult_visits_Full$recidFlag == 0,]
  # Keep second attempts
  seconds <- mult_visits_NOR[grep("Visit2", mult_visits_NOR$nwid_new),]
  
  ids_checked <- unique(duplicatedObs$nwid_new)
  
  if(length(ids_checked) > 0){
    adjusted <- data.frame()
    for(i in 1:length(ids_checked)){
      ind_set <- mult_visits[grep(ids_checked[i], mult_visits$wcid),]
      # now, go through this ind_set and determine which observations I should keep,
      # which are the relevant visits for recidivism, the most recent
      # which of these visits according to exit clean was closest to the recid date,
      # but there might be multiple cases of recidivism
      recid_dates <- unique(ind_set$recidDate)
      id_rows <- data.frame()
      for(r in 1:length(recid_dates)){
        temp <- ind_set[ind_set$recidDate == recid_dates[r],]
        # find the visits that ended before recidivism
        temp <- temp[which(as.numeric(temp$exitClean) <= as.numeric(recid_dates[r])),]
        # find the exit closest to the recid_date
        temp <- temp[which.min(as.numeric(recid_dates[r]) - as.numeric(temp$exitClean)),]
        id_rows <- rbind(id_rows, temp)
      }
      adjusted <- rbind(adjusted, id_rows)
    }
    
    seconds_complete <- rbind(seconds, adjusted)
    
    # now, I need to get a data frame completely devoid of those duplicate observations
    single_visits <- new_df[-c(grep("Visit", new_df$wcid)),]
    
    dv <- rbind(single_visits, seconds_complete)
  } else {
    dv <- new_df
  }

  # for right censoring, recode NA's for recidDate as 20090904
  dv[is.na(dv$recidDate),]$recidDate <- "20090904"
  
  library(survival)
  dv$exitDays <- difftime(as.Date(dv$exitClean, format = "%Y%m%d"), as.Date("2001-01-05"), units = "days") # days
  dv$recidDays <- difftime(as.Date(dv$recidDate, format = "%Y%m%d"), as.Date("2001-01-05"), units = "days") # days 
  
  dv$gap <- c(dv$recidDays-dv$exitDays)
  
  dv <- with(dv, data.frame(
    exitClean = dv$exitClean,
    enterClean = dv$enterClean,
    success = success,
    nwid_new = wcid,
    gap = gap,
    recidFlag = recidFlag,
    strata = strata,
    age = age,
    lsi = lsi,
    black = black
  ))
  
  dv <- na.omit(dv)
  
  # remove those who recid, no expectation that TC should inform after first recid following
  
  dv <- dv[dv$strata <1,]
  ### get network covariates
  library(tnam)
  
  recid_list <- dv$recidFlag
  names(recid_list) <- dv$nwid_new
  recid_list <- list(t1 = recid_list)
  
  success_list <- dv$success
  names(success_list) <- dv$nwid_new
  success_list <- list(t1 = success_list)
  
  # create a reverse coded version
  
  gap_list <- as.numeric(dv$gap)
  names(gap_list) <- dv$nwid_new
  gap_list <- list(t1 = gap_list)
  
  age_list <- as.numeric(dv$age)
  names(age_list) <- dv$nwid_new
  age_list <- list(t1 = age_list)
  
  black_list <- dv$black
  names(black_list) <- dv$nwid_new
  black_list <- list(t1 = black_list)
  
  lsi_list <- dv$lsi
  names(lsi_list) <- dv$nwid_new
  lsi_list <- list(t1 = lsi_list)
  
  all_1s <- rep(1, times = nrow(dv))
  names(all_1s) <- dv$nwid_new
  all_1s <- list(t1 = all_1s)
  
  library(lubridate)
  finish <- ymd(dv$exitClean)
  start <- ymd(dv$enterClean)
  elapsed <-  difftime(finish,start,units="days")
  
  names(elapsed) <- dv$wcid
  days_list <- list(t1 = elapsed)
  
  tnam_out <- tnam(recid_list ~
                     netlag(success_list, correct_adj, pathdist = 1, coefname = "Correct_1") +
                     netlag(success_list, correct_adj, pathdist = 2, coefname = "Correct_2") +
                     weightlag(success_list, correct_adj, coefname = "WeightCorrect_1") +
                     centrality(correct_adj, type = "indegree", coefname = "Correct_InCent") +
                     centrality(correct_adj, type = "outdegree", coefname = "Correct_OutCent") +
                     centrality(correct_adj, type = "eigenvector", coefname = "Correct_EigenVectorCent") +
                     structsim(success_list, correct_adj, method = "euclidean", coefname = "Correct_StructSim_Euclidean")+
                     structsim(success_list, correct_adj, method = "jaccard", coefname = "Correct_StructSim_Jaccard")+
                     clustering(correct_adj, coefname = "Correct_Clustering_In") +
                     netlag(success_list, affirm_adj, pathdist = 1, coefname = "Affirm_1") +
                     netlag(success_list, affirm_adj, pathdist = 2, coefname = "Affirm_2") +
                     weightlag(success_list, affirm_adj, coefname = "WeightAffirm_1") +
                     centrality(affirm_adj, type = "indegree", coefname = "Affirm_InCent") +
                     centrality(affirm_adj, type = "outdegree", coefname = "Affirm_OutCent") +
                     centrality(affirm_adj, type = "eigenvector", coefname = "Affirm_EigenVectorCent") +
                     clustering(affirm_adj, coefname = "Affirm_Clustering_In") +
                     structsim(success_list, affirm_adj, method = "euclidean", coefname = "Affirm_StructSim_Euclidean")+
                     structsim(success_list, affirm_adj, method = "jaccard", coefname = "Affirm_StructSim_Jaccard")+
                     covariate(gap_list, coefname = "Gap") +
                     covariate(age_list, coefname = "Age") +
                     covariate(black_list, coefname = "Black") +
                     covariate(lsi_list, coefname = "LSI") +
                     covariate(days_list, coefname = "Days") +
                     netlag(all_1s, affirm_adj, pathdist = 1, coefname = "Affirm_1Paths") +
                     netlag(all_1s, affirm_adj, pathdist = 2, coefname = "Affirm_2Paths") +
                     netlag(all_1s, affirm_adj, pathdist = 3, coefname = "Affirm_3Paths") +
                     netlag(all_1s, affirm_adj, pathdist = 4, coefname = "Affirm_4Paths") + 
                     netlag(all_1s, correct_adj, pathdist = 1, coefname = "Correct_1Paths") +
                     netlag(all_1s, correct_adj, pathdist = 2, coefname = "Correct_2Paths") +
                     netlag(all_1s, correct_adj, pathdist = 3, coefname = "Correct_3Paths") +
                     netlag(all_1s, correct_adj, pathdist = 4, coefname = "Correct_4Paths") +
                     centrality(affirm_adj, type = "closeness", coefname = "Affirm_Closeness") +
                     centrality(correct_adj, type = "closeness", coefname = "Correct_Closeness") +
                     covariate(success_list, coefname = "Success"),
                   re.node = FALSE, 
                   time.linear = FALSE)
  
  dv <- tnam_out$model
  colnames(dv) <- c("recidFlag", "Correct1", "Correct2", "WeightCorrect", "CorrectIn", "CorrectOut", "CorrectEigen", "CorrectStructSim_Euclidean", "CorrectStructSim_Jaccard", "CorrectClustering",
                    "Affirm1", "Affirm2", "WeightAffirm", "AffirmIn", "AffirmOut", "AffirmEigen", "AffirmClustering", "AffirmStructSim_Euclidean", "AffirmStructSim_Jaccard", 
                    "Gap", "Age", "Black", "LSI", "Days", 
                    "Affirm_1Paths", "Affirm_2Paths", "Affirm_3Paths", "Affirm_4Paths", 
                    "Correct_1Paths", "Correct_2Paths", "Correct_3Paths", "Correct_4Paths",
                    "AffirmCloseness", "CorrectCloseness", "Success")
  
  NW <- dv
  
  
  # Get days difference between
  surv_out <- Surv(time = as.numeric(dv$Gap),
                   event = dv$recidFlag)
  
  save(NW, file = paste0(WD,"/Data/TC_NW_TNAMData.RData"))
} else {
  load(paste0(WD,"/Data/TC_NW_TNAMData.RData"))
}
