describe <- function(variable = NULL, type = c("continuous", "binary"), varname = NULL){
  require(ggplot2)
  require(reshape2)
  if(type == "continuous"){
    dist <- summary(variable)
    missingness <- table(is.na(variable))
    plot_df <- melt(variable)
    summary_plot <- ggplot(data = plot_df, aes(x = value)) +
      geom_density(fill = "firebrick4", color = "black") +
      theme_bw() +
      xlab("Variable Value") +
      ylab("Density") +
      ggtitle(varname)
      theme(legend.position = c(0.2, 0.8), 
            axis.text=element_text(size=12),
            axis.title=element_text(size=14,face="bold")) 
  }
  
  if(type == "binary"){
    dist <- table(variable)
    missingness <- table(is.na(variable))
    plot_df <- melt(variable)
    summary_plot <- ggplot(data = plot_df, aes(x = value)) +
      geom_histogram(fill = "firebrick4", color = "black", bins = length(dist)) +
      theme_bw() +
      xlab("Variable Value") +
      ylab("Count") +
      ggtitle(varname) +
      scale_x_discrete(limits=c(1:length(dist)-1)) +
      theme(legend.position = c(0.2, 0.8), 
            axis.text=element_text(size=12),
            axis.title=element_text(size=14,face="bold")) 
  }
  return(list(dist = dist, missingness = missingness, summary_plot = summary_plot))
}
