# WCC_Replication.R
# THis script, when sourced from WCC_Run.R produces all necessary display items produced in WCC.  
# Aut: Benjamin Campbell
# Date: 03/07/2019

######################
# Bootstrapped Cox
# TNAMs
######################
##############
# West Central
##############
cat("Starting data compilation... \n")
write.wcm.data = recompile
source(paste0(WD,"/Data/WCM_DataPrep.R"))
cat("First dataset compiled/imported. \n")

write.wcf.data = recompile
source(paste0(WD,"/Data/WCF_DataPrep.R"))
cat("Second dataset compiled/imported. \n")
###########
# Northwest
###########

write.nw.data = recompile
source(paste0(WD,"/Data/NW_DataPrep.R"))
cat("Third dataset compiled/imported. \n")
###########
# MonDay
###########

write.mdm.data = recompile
source(paste0(WD,"/Data/MDM_DataPrep.R"))
cat("Fourth dataset compiled/imported. \n")

write.mdf.data = recompile
source(paste0(WD,"/Data/MDF_DataPrep.R"))
cat("Fifth dataset compiled/imported. \n")

library(survival)

WCM <- WCM[WCM$Gap > 0,]
surv_out_wcm <- Surv(WCM$Gap, WCM$recidFlag)
WCM$id <- paste0("ID", 1:nrow(WCM))

WCF <- WCF[WCF$Gap > 0,]
surv_out_wcf <- Surv(WCF$Gap, WCF$recidFlag)
WCF$id <- paste0("ID", 1:nrow(WCF))

MDM <- MDM[MDM$Gap > 0,]
surv_out_mdm <- Surv(MDM$Gap, MDM$recidFlag)
MDM$id <- paste0("ID", 1:nrow(MDM))

MDF <- MDF[MDF$Gap > 0,]
surv_out_mdf <- Surv(MDF$Gap, MDF$recidFlag)
MDF$id <- paste0("ID", 1:nrow(MDF))

NW <- NW[NW$Gap > 0,]
surv_out_nw <- Surv(NW$Gap, NW$recidFlag)
NW$id <- paste0("ID", 1:nrow(NW))


cat("Estimating bootstrapped models... \n")
n_boot_reps <- 25000

library(survival)

set.seed(12345)
boot_wcm <- function(i = NULL){
  pct_sampled <- 0.9
  indices <- sample(1:nrow(WCM), round(nrow(WCM)*pct_sampled), replace = FALSE)
  surv_out_wcm_boot <- Surv(WCM$Gap[indices], WCM$recidFlag[indices])
  one_it <- coxph(surv_out_wcm_boot ~
                    I(log(Age+1))+
                    #    I(log(LSI+1)) +
                    Black +
                    I(log(Days+1)) +
                    AffirmClustering +
                    #       Affirm1 +
                    #        Affirm2 +
                    #  AffirmEigen +
                    frailty(id, distribution="gamma", method="em", eps = 0.005),
                  method = c("efron"),
                  x = TRUE, 
                  y = TRUE,
                  data = WCM[indices,])
  print(i)
  return(one_it$coefficients)
}

library(parallel)
wcm_out <- mclapply(as.list(1:n_boot_reps), boot_wcm, mc.cores = 3)
wcm_coefs <- do.call(rbind,wcm_out)

wcm_coefs <- na.omit(wcm_coefs)
wcm_summary <- data.frame(
  means = c(mean(wcm_coefs[,1]), mean(wcm_coefs[,2]), mean(wcm_coefs[,3],), mean(wcm_coefs[,4])),
  sd = c(sd(wcm_coefs[,1]), sd(wcm_coefs[,2]), sd(wcm_coefs[,3]), sd(wcm_coefs[,4]))
)
wcm_summary$low_95 <- with(wcm_summary, means-(1.96*sd))
wcm_summary$high_95 <- with(wcm_summary, means+(1.96*sd))
rownames(wcm_summary) <- c("age", "black", "days", "affirmClustering")
cat("First model completed. \n")

set.seed(12345)
boot_wcf <- function(i = NULL){
  pct_sampled <- 0.9
  indices <- sample(1:nrow(WCF), round(nrow(WCF)*pct_sampled), replace = FALSE)
  surv_out_wcf_boot <- Surv(WCF$Gap[indices], WCF$recidFlag[indices])
  one_it <- coxph(surv_out_wcf_boot ~
                    I(log(Age+1))+
                    #    I(log(LSI+1)) +
                    Black +
                    I(log(Days+1)) +
                    AffirmClustering +
                    #       Affirm1 +
                    #        Affirm2 +
                    #  AffirmEigen +
                    frailty(id, distribution="gamma", method="em", eps = 0.005),
                  method = c("efron"),
                  x = TRUE, 
                  y = TRUE,
                  data = WCF[indices,])
  print(i)
  return(one_it$coefficients)
}

wcf_out <- mclapply(as.list(1:n_boot_reps), boot_wcf, mc.cores = 3)
wcf_coefs <- do.call(rbind,wcf_out)

wcf_coefs <- na.omit(wcf_coefs)
wcf_summary <- data.frame(
  means = c(mean(wcf_coefs[,1]), mean(wcf_coefs[,2]), mean(wcf_coefs[,3],), mean(wcf_coefs[,4])),
  sd = c(sd(wcf_coefs[,1]), sd(wcf_coefs[,2]), sd(wcf_coefs[,3]), sd(wcf_coefs[,4]))
)
wcf_summary$low_95 <- with(wcf_summary, means-(1.96*sd))
wcf_summary$high_95 <- with(wcf_summary, means+(1.96*sd))
rownames(wcf_summary) <- c("age", "black", "days", "affirmClustering")
cat("Second model completed. \n")

set.seed(12345)
boot_mdm <- function(i = NULL){
  pct_sampled <- 0.9
  indices <- sample(1:nrow(MDM), round(nrow(MDM)*pct_sampled), replace = FALSE)
  surv_out_mdm_boot <- Surv(MDM$Gap[indices], MDM$recidFlag[indices])
  one_it <- coxph(surv_out_mdm_boot ~
                    I(log(Age+1))+
                    #    I(log(LSI+1)) +
                    Black +
                    I(log(Days+1)) +
                    AffirmClustering +
                    #       Affirm1 +
                    #        Affirm2 +
                    #  AffirmEigen +
                    frailty(id, distribution="gamma", method="em", eps = 0.005),
                  method = c("efron"),
                  x = TRUE, 
                  y = TRUE,
                  data = MDM[indices,])
  print(i)
  return(one_it$coefficients)
}

mdm_out <- mclapply(as.list(1:n_boot_reps), boot_mdm, mc.cores = 3)
mdm_coefs <- do.call(rbind,mdm_out)

mdm_coefs <- na.omit(mdm_coefs)
mdm_summary <- data.frame(
  means = c(mean(mdm_coefs[,1]), mean(mdm_coefs[,2]), mean(mdm_coefs[,3],), mean(mdm_coefs[,4])),
  sd = c(sd(mdm_coefs[,1]), sd(mdm_coefs[,2]), sd(mdm_coefs[,3]), sd(mdm_coefs[,4]))
)
mdm_summary$low_95 <- with(mdm_summary, means-(1.96*sd))
mdm_summary$high_95 <- with(mdm_summary, means+(1.96*sd))
rownames(mdm_summary) <- c("age", "black", "days", "affirmClustering")

cat("Third model completed. \n")
set.seed(12345)
boot_mdf <- function(i = NULL){
  pct_sampled <- 0.9
  indices <- sample(1:nrow(MDF), round(nrow(MDF)*pct_sampled), replace = FALSE)
  surv_out_mdf_boot <- Surv(MDF$Gap[indices], MDF$recidFlag[indices])
  one_it <- coxph(surv_out_mdf_boot ~
                    I(log(Age+1))+
                    #    I(log(LSI+1)) +
                    Black +
                    I(log(Days+1)) +
                    AffirmClustering +
                    #       Affirm1 +
                    #        Affirm2 +
                    #  AffirmEigen +
                    frailty(id, distribution="gamma", method="em", eps = 0.005),
                  method = c("efron"),
                  x = TRUE, 
                  y = TRUE,
                  data = MDF[indices,])
  print(i)
  return(one_it$coefficients)
}

mdf_out <- mclapply(as.list(1:n_boot_reps), boot_mdf, mc.cores = 3)
mdf_coefs <- do.call(rbind,mdf_out)

mdf_coefs <- na.omit(mdf_coefs)
mdf_summary <- data.frame(
  means = c(mean(mdf_coefs[,1]), mean(mdf_coefs[,2]), mean(mdf_coefs[,3],), mean(mdf_coefs[,4])),
  sd = c(sd(mdf_coefs[,1]), sd(mdf_coefs[,2]), sd(mdf_coefs[,3]), sd(mdf_coefs[,4]))
)
mdf_summary$low_95 <- with(mdf_summary, means-(1.96*sd))
mdf_summary$high_95 <- with(mdf_summary, means+(1.96*sd))
rownames(mdf_summary) <- c("age", "black", "days", "affirmClustering")

cat("Fourth model completed. \n")
set.seed(12345)
boot_nw <- function(i = NULL){
  pct_sampled <- 0.9
  indices <- sample(1:nrow(NW), round(nrow(NW)*pct_sampled), replace = FALSE)
  surv_out_nw_boot <- Surv(NW$Gap[indices], NW$recidFlag[indices])
  one_it <- coxph(surv_out_nw_boot ~
                    I(log(Age+1))+
                    #    I(log(LSI+1)) +
                    Black +
                    I(log(Days+1)) +
                    AffirmClustering +
                    #       Affirm1 +
                    #        Affirm2 +
                    #  AffirmEigen +
                    frailty(id, distribution="gamma", method="em", eps = 0.005),
                  method = c("efron"),
                  x = TRUE, 
                  y = TRUE,
                  data = NW[indices,])
  print(i)
  return(one_it$coefficients)
}

nw_out <- mclapply(as.list(1:n_boot_reps), boot_nw, mc.cores = 3)
nw_coefs <- do.call(rbind,nw_out)

nw_coefs <- na.omit(nw_coefs)
nw_summary <- data.frame(
  means = c(mean(nw_coefs[,1]), mean(nw_coefs[,2]), mean(nw_coefs[,3],), mean(nw_coefs[,4])),
  sd = c(sd(nw_coefs[,1]), sd(nw_coefs[,2]), sd(nw_coefs[,3]), sd(nw_coefs[,4]))
)
nw_summary$low_95 <- with(nw_summary, means-(1.96*sd))
nw_summary$high_95 <- with(nw_summary, means+(1.96*sd))
rownames(nw_summary) <- c("age", "black", "days", "affirmClustering")

coef_lists <- list(wcm_coefs = wcm_summary,
                   wcf_coefs = wcf_summary,
                   mdm_coefs = mdm_summary,
                   mdf_coefs = mdf_summary,
                   nw_coefs = nw_summary)
cat("Second model completed. Writing Table 2. \n")
sink(paste0(WD,"/Output/Table2.txt"))
coef_lists
sink()


# WCM
WCM$Age_log <- log(WCM$Age+1)
WCM$Days_log <- log(WCM$Days+1)
wcm_fit <- coxph(Surv(WCM$Gap, WCM$recidFlag) ~
                   Age_log +
                   #    I(log(LSI+1)) +
                   Black +
                   Days_log +
                   AffirmClustering +
                   #       Affirm1 +
                   #        Affirm2 +
                   #  AffirmEigen +
                   frailty(id, distribution="gamma", method="em", eps = 0.005),
                 method = c("efron"),
                 x = TRUE, 
                 y = TRUE,
                 data = WCM)

# WCF
WCF$Age_log <- log(WCF$Age+1)
WCF$Days_log <- log(WCF$Days+1)
wcf_fit <- coxph(Surv(WCF$Gap, WCF$recidFlag) ~
                   Age_log +
                   #    I(log(LSI+1)) +
                   Black +
                   Days_log +
                   AffirmClustering +
                   #       Affirm1 +
                   #        Affirm2 +
                   #  AffirmEigen +
                   frailty(id, distribution="gamma", method="em", eps = 0.005),
                 method = c("efron"),
                 x = TRUE, 
                 y = TRUE,
                 data = WCF)

# MDM
MDM$Age_log <- log(MDM$Age+1)
MDM$Days_log <- log(MDM$Days+1)
mdm_fit <- coxph(Surv(MDM$Gap, MDM$recidFlag) ~
                   Age_log +
                   #    I(log(LSI+1)) +
                   Black +
                   Days_log +
                   AffirmClustering +
                   #       Affirm1 +
                   #        Affirm2 +
                   #  AffirmEigen +
                   frailty(id, distribution="gamma", method="em", eps = 0.005),
                 method = c("efron"),
                 x = TRUE, 
                 y = TRUE,
                 data = MDM)

# MDF
MDF$Age_log <- log(MDF$Age+1)
MDF$Days_log <- log(MDF$Days+1)
mdf_fit <- coxph(Surv(MDF$Gap, MDF$recidFlag) ~
                   Age_log +
                   #    I(log(LSI+1)) +
                   Black +
                   Days_log +
                   AffirmClustering +
                   #       Affirm1 +
                   #        Affirm2 +
                   #  AffirmEigen +
                   frailty(id, distribution="gamma", method="em", eps = 0.005),
                 method = c("efron"),
                 x = TRUE, 
                 y = TRUE,
                 data = MDF)

# NW
NW$Age_log <- log(NW$Age+1)
NW$Days_log <- log(NW$Days+1)
nw_fit <- coxph(Surv(NW$Gap, NW$recidFlag) ~
                  Age_log +
                  #    I(log(LSI+1)) +
                  Black +
                  Days_log +
                  AffirmClustering +
                  #       Affirm1 +
                  #        Affirm2 +
                  #  AffirmEigen +
                  frailty(id, distribution="gamma", method="em", eps = 0.005),
                method = c("efron"),
                x = TRUE, 
                y = TRUE,
                data = NW)


sink(paste0(WD,"/Output/TGTestResults.txt"))
cat("WCM \n")
cox.zph(wcm_fit)
cat("WCF \n")
cox.zph(wcf_fit)
cat("MDM \n")
cox.zph(mdm_fit)
cat("MDF \n")
cox.zph(mdf_fit)
cat("NW \n")
cox.zph(nw_fit)
sink()

#devtools::install_github("benjamin-w-campbell/coxtools")
library(coxtools)

# make pred prob plots
# MDM = facility 1 men
# MDF = facility 1 women
# NW = facility 2
# WCM = facility 3 male
# WCF = facility 3 female

cat("Estimating predicted probabilities, Figure 1. \n")
wcm_plot <- plot_predicted_probs(wcm_fit, var ="AffirmClustering", time = 365, 
                                 xaxis_label = "Clustering Coefficient",
                                 yaxis_label = "Probability of Not Recidivating",
                                 title = "Facility 3, Male",
                                 seed = 1)

wcf_plot <- plot_predicted_probs(wcf_fit, "AffirmClustering", time = 365, 
                                 xaxis_label = "Clustering Coefficient",
                                 yaxis_label = "Probability of Not Recidivating",
                                 title = "Facility 3, Female",
                                 seed = 1)

mdm_plot <- plot_predicted_probs(mdm_fit, "AffirmClustering", time = 365, 
                                 xaxis_label = "Clustering Coefficient",
                                 yaxis_label = "Probability of Not Recidivating",
                                 title = "Facility 1, Male",
                                 seed = 1)

mdf_plot <- plot_predicted_probs(mdf_fit, "AffirmClustering", time = 365, 
                                 xaxis_label = "Clustering Coefficient",
                                 yaxis_label = "Probability of Not Recidivating",
                                 title = "Facility 1, Female",
                                 seed = 1)

nwm_plot <- plot_predicted_probs(nw_fit, "AffirmClustering", time = 365, 
                                 xaxis_label = "Clustering Coefficient",
                                 yaxis_label = "Probability of Not Recidivating",
                                 title = "Facility 2, Male",
                                 seed = 1)


plotlist <- list(mdm_plot, mdf_plot, nwm_plot, wcm_plot, wcf_plot)

pdf(paste0(WD,"/Output/Figure1.pdf"), height = 10, width = 10)
Rmisc::multiplot(plotlist=plotlist, cols = 3)
dev.off()


vars <- c("recidFlag", "Gap", "Success", "LSI", "Days", "Age", "Black", "Affirm1", "Affirm2",
          "WeightAffirm", "AffirmIn", "AffirmOut", "AffirmClustering", "Affirm_1Paths", 
          "Affirm_2Paths", "Affirm_3Paths")

colnames(MDM)[27:30] <- c("Affirm_1Paths", "Affirm_2Paths", "Affirm_3Paths", "Affirm_4Paths") 
colnames(MDF)[27:30] <- c("Affirm_1Paths", "Affirm_2Paths", "Affirm_3Paths", "Affirm_4Paths") 

data_objects <- list(WCM, WCF, NW, MDM, MDF)

cat("Writing descriptive statistics, Table 1 \n")
source(paste0(WD,"/Data/describe.R"))
plot_list <- list()
summary_list <- list()
for(i in 1:length(data_objects)){
  dat <- data_objects[[i]]
  dat_plots <- list()
  dat_summary  <- list()
  for(j in 1:length(vars)){
    vec <- dat[,vars[j]]
    if(vars[j] %in% c("recidFlag", "Success", "Black")){
      out <- describe(variable = vec, type = "binary", varname = vars[j])
    } else {
      out <- describe(variable = vec, type = "continuous", varname = vars[j])
    }
    dat_plots[[j]] <- out$summary_plot 
    dat_summary[[j]] <- out$dist
  }
  plot_list[[i]] <- dat_plots
  summary_list[[i]] <- dat_summary
}

# Now, write the summary to an output file
dataset_names <- c("West Central, Men", "West Central, Female", "Northwest", "Monday, Men", "Monday, Female")

sink(paste0(WD,"/Output/Table1.txt"))
cat("###################################################### \n")
cat("Summary statistics per dataset\n")
cat("###################################################### \n")
for(i in 1:length(summary_list)){
  summary_i <- summary_list[[i]]
  names(summary_i) <- vars
  
  cat("###################################################### \n")
  cat(dataset_names[i], "\n")
  cat("###################################################### \n")
  print(summary_i)
}
sink()

cat("Finished! \n")